@@ -9,6 +9,8 @@ use std::cmp::Ordering::{self, *};
 use num_bigint::{BigInt, BigUint};
 #[cfg(feature = "num-rational")]
 use num_rational::Ratio;
+#[cfg(feature = "num-complex")]
+use num_complex::Complex;
 
 #[derive(Clone, Debug)]
 #[allow(non_camel_case_types)]
@@ -17,9 +19,9 @@ enum N {
     i8(i8), i16(i16), i32(i32), i64(i64), i128(i128), isize(isize),
     f32(f32), f64(f64),
     #[cfg(feature = "num-bigint")]
-    biguint(BigUint),
+    ubig(BigUint),
     #[cfg(feature = "num-bigint")]
-    bigint(BigInt),
+    ibig(BigInt),
     #[cfg(feature = "num-rational")]
     r8(Ratio<i8>),
     #[cfg(feature = "num-rational")]
@@ -30,6 +32,12 @@ enum N {
     r64(Ratio<i64>),
     #[cfg(feature = "num-rational")]
     rsize(Ratio<isize>),
+    #[cfg(all(feature = "num-bigint", feature = "num-rational"))]
+    rbig(Ratio<BigInt>),
+    #[cfg(feature = "num-complex")]
+    c32(Complex<f32>),
+    #[cfg(feature = "num-complex")]
+    c64(Complex<f64>),
 }
 
 macro_rules! repeat_arms {
@@ -41,9 +49,9 @@ macro_rules! repeat_arms {
             N::i64($v) => $arm, N::i128($v) => $arm, N::isize($v) => $arm,
             N::f32($v) => $arm, N::f64($v) => $arm,
             #[cfg(feature = "num-bigint")]
-            N::biguint($v) => $arm,
+            N::ubig($v) => $arm,
             #[cfg(feature = "num-bigint")]
-            N::bigint($v) => $arm,
+            N::ibig($v) => $arm,
             #[cfg(feature = "num-rational")]
             N::r8($v) => $arm,
             #[cfg(feature = "num-rational")]
@@ -54,6 +62,12 @@ macro_rules! repeat_arms {
             N::r64($v) => $arm,
             #[cfg(feature = "num-rational")]
             N::rsize($v) => $arm,
+            #[cfg(all(feature = "num-bigint", feature = "num-rational"))]
+            N::rbig($v) => $arm,
+            #[cfg(feature = "num-complex")]
+            N::c32($v) => $arm,
+            #[cfg(feature = "num-complex")]
+            N::c64($v) => $arm,
         }
     };
 }
@@ -397,22 +411,23 @@ fn expand_equiv_class(cls: &[N]) -> Vec<N> {
             N::isize(v) => ret.push(N::isize(*v)),
             N::f32(v) => ret.extend_from_slice(&[N::f32(*v), N::f64(*v as f64)]),
             N::f64(v) => ret.push(N::f64(*v)),
+            #[cfg(any(feature = "num-rational", feature = "num-bigint", feature = "num-complex"))]
             _ => {}
         }
 
         // size extension for bigints
         #[cfg(feature = "num-bigint")]
         match e {
-            N::u8(v) => ret.push(N::biguint(BigUint::from(*v))),
-            N::u16(v) => ret.push(N::biguint(BigUint::from(*v))),
-            N::u32(v) => ret.push(N::biguint(BigUint::from(*v))),
-            N::u64(v) => ret.push(N::biguint(BigUint::from(*v))),
-            N::u128(v) => ret.push(N::biguint(BigUint::from(*v))),
-            N::i8(v) => ret.push(N::bigint(BigInt::from(*v))),
-            N::i16(v) => ret.push(N::bigint(BigInt::from(*v))),
-            N::i32(v) => ret.push(N::bigint(BigInt::from(*v))),
-            N::i64(v) => ret.push(N::bigint(BigInt::from(*v))),
-            N::i128(v) => ret.push(N::bigint(BigInt::from(*v))),
+            N::u8(v) => ret.push(N::ubig(BigUint::from(*v))),
+            N::u16(v) => ret.push(N::ubig(BigUint::from(*v))),
+            N::u32(v) => ret.push(N::ubig(BigUint::from(*v))),
+            N::u64(v) => ret.push(N::ubig(BigUint::from(*v))),
+            N::u128(v) => ret.push(N::ubig(BigUint::from(*v))),
+            N::i8(v) => ret.push(N::ibig(BigInt::from(*v))),
+            N::i16(v) => ret.push(N::ibig(BigInt::from(*v))),
+            N::i32(v) => ret.push(N::ibig(BigInt::from(*v))),
+            N::i64(v) => ret.push(N::ibig(BigInt::from(*v))),
+            N::i128(v) => ret.push(N::ibig(BigInt::from(*v))),
             _ => {}
         }
 
@@ -525,6 +540,16 @@ fn test_nan() {
             assert_cmp(i, &N::f64(f64::NAN), None);
             assert_cmp(&N::f32(f32::NAN), i, None);
             assert_cmp(&N::f64(f64::NAN), i, None);
+
+            #[cfg(feature = "num-complex")]
+            {
+                assert_cmp(i, &N::c32(Complex::new(f32::NAN, 0.)), None);
+                assert_cmp(i, &N::c32(Complex::new(0., f32::NAN)), None);
+                assert_cmp(i, &N::c32(Complex::new(f32::NAN, f32::NAN)), None);
+                assert_cmp(&N::c32(Complex::new(f32::NAN, 0.)), i, None);
+                assert_cmp(&N::c32(Complex::new(0., f32::NAN)), i, None);
+                assert_cmp(&N::c32(Complex::new(f32::NAN, f32::NAN)), i, None);
+            }
         }
     }
 
@@ -533,6 +558,22 @@ fn test_nan() {
     assert_cmp(&N::f32(f32::NAN), &N::f64(f64::NAN), None);
     assert_cmp(&N::f64(f64::NAN), &N::f32(f32::NAN), None);
     assert_cmp(&N::f64(f64::NAN), &N::f64(f64::NAN), None);
+
+    #[cfg(feature = "num-complex")]
+    {
+        let cnan0 = N::c32(Complex::new(f32::NAN, 0.));
+        let c0nan = N::c32(Complex::new(0., f32::NAN));
+        let cnannan = N::c32(Complex::new(f32::NAN, f32::NAN));
+        assert_cmp(&cnan0, &cnan0, None);
+        assert_cmp(&cnan0, &c0nan, None);
+        assert_cmp(&cnan0, &cnannan, None);
+        assert_cmp(&c0nan, &cnan0, None);
+        assert_cmp(&c0nan, &c0nan, None);
+        assert_cmp(&c0nan, &cnannan, None);
+        assert_cmp(&cnannan, &cnan0, None);
+        assert_cmp(&cnannan, &c0nan, None);
+        assert_cmp(&cnannan, &cnannan, None);
+    }
 }
 
 
@@ -548,7 +589,7 @@ fn test_hash() {
 
 #[test]
 #[cfg(feature = "num-rational")]
-fn test_rational_using_prim() {
+fn test_rational_using_primitives() {
     fn expand_equiv_class_ratio(cls: &[N]) -> Vec<N> {
         let mut ret = Vec::new();
     
@@ -567,8 +608,13 @@ fn test_rational_using_prim() {
                 N::i16(v) => ret.extend_from_slice(&[N::i16(*v), N::r16(Ratio::from(*v))]),
                 N::i32(v) => ret.extend_from_slice(&[N::i32(*v), N::r32(Ratio::from(*v))]),
                 N::i64(v) => ret.extend_from_slice(&[N::i64(*v), N::r64(Ratio::from(*v))]),
+                #[cfg(not(feature = "num-bigint"))]
                 N::i128(v) => ret.push(N::i128(*v)),
+                #[cfg(feature = "num-bigint")]
+                N::i128(v) => ret.extend_from_slice(&[N::i128(*v), N::rbig(Ratio::from(BigInt::from(*v)))]),
                 N::isize(v) => ret.extend_from_slice(&[N::isize(*v), N::rsize(Ratio::from(*v))]),
+                #[cfg(feature = "num-bigint")]
+                N::ibig(v) => ret.extend_from_slice(&[N::ibig(v.clone()), N::rbig(Ratio::from(v.clone()))]),
                 _ => {}
             }
         }
@@ -601,6 +647,9 @@ fn test_rational_using_prim() {
 #[test]
 #[cfg(feature = "num-rational")]
 fn test_rational() {
+    #[cfg(feature = "num-bigint")]
+    let big = || BigInt::from(1u8) << 200u8;
+    
     // additional test cases for rational numbers: (numer, denom, float value)
     let ratio_coeffs = [
         (N::i8(-2), N::i8(1), Some(N::f32(-2.))),
@@ -608,7 +657,11 @@ fn test_rational() {
         // near -1
         (N::i32(i32::MIN), N::i32(i32::MAX), None),
         (N::i64(i64::MIN), N::i64(i64::MAX), None),
+        #[cfg(feature = "num-bigint")]
+        (N::ibig(-big()), N::ibig(big() - 1), None),
         (N::i8(-1), N::i8(1), Some(N::f32(-1.))),
+        #[cfg(feature = "num-bigint")]
+        (N::ibig(-big()), N::ibig(big() + 1), None),
         (N::i64(i64::MIN + 2), N::i64(i64::MAX), None),
         (N::i32(i32::MIN + 2), N::i32(i32::MAX), None),
 
@@ -622,7 +675,11 @@ fn test_rational() {
         // near 0
         (N::i32(-1), N::i32(i32::MAX), None),
         (N::i64(-1), N::i64(i64::MAX), None),
+        #[cfg(feature = "num-bigint")]
+        (N::ibig(-BigInt::from(1u8)), N::ibig(big()), None),
         (N::i8(0), N::i8(1), Some(N::f32(0.))),
+        #[cfg(feature = "num-bigint")]
+        (N::ibig(BigInt::from(1u8)), N::ibig(big()), None),
         (N::i64(1), N::i64(i64::MAX), None),
         (N::i32(1), N::i32(i32::MAX), None),
 
@@ -636,40 +693,77 @@ fn test_rational() {
         // near 1
         (N::i32(i32::MAX-1), N::i32(i32::MAX), None),
         (N::i64(i64::MAX-1), N::i64(i64::MAX), None),
+        #[cfg(feature = "num-bigint")]
+        (N::ibig(big()), N::ibig(big() + 1), None),
         (N::i8(1), N::i8(1), Some(N::f32(1.))),
+        #[cfg(feature = "num-bigint")]
+        (N::ibig(big()), N::ibig(big() - 1), None),
         (N::i64(i64::MAX), N::i64(i64::MAX-1), None),
         (N::i32(i32::MAX), N::i32(i32::MAX-1), None),
 
         (N::i8(2), N::i8(1), Some(N::f32(2.))),
     ];
 
-    fn expand_equiv_class_ratio(num: &N, den: &N) -> Vec<N> {
+    fn expand_equiv_class_ratio(coeffs: &(N, N, Option<N>)) -> Vec<N> {
         let mut ret = Vec::new();
 
-        match (num, den) {
-            (N::i8(num), N::i8(den)) => ret.extend_from_slice(&[
+        #[cfg(not(feature = "num-bigint"))]
+        match coeffs {
+            (N::i8(num), N::i8(den), _) => ret.extend_from_slice(&[
                 N::r8(Ratio::new(*num, *den)),
                 N::r16(Ratio::new(*num as i16, *den as i16)),
                 N::r32(Ratio::new(*num as i32, *den as i32)),
                 N::r64(Ratio::new(*num as i64, *den as i64))]),
-            (N::i16(num), N::i16(den)) => ret.extend_from_slice(&[
+            (N::i16(num), N::i16(den), _) => ret.extend_from_slice(&[
                 N::r16(Ratio::new(*num as i16, *den as i16)),
                 N::r32(Ratio::new(*num as i32, *den as i32)),
                 N::r64(Ratio::new(*num as i64, *den as i64))]),
-            (N::i32(num), N::i32(den)) => ret.extend_from_slice(&[
+            (N::i32(num), N::i32(den), _) => ret.extend_from_slice(&[
                 N::r32(Ratio::new(*num as i32, *den as i32)),
                 N::r64(Ratio::new(*num as i64, *den as i64))]),
-            (N::i64(num), N::i64(den)) => ret.extend_from_slice(&[
+            (N::i64(num), N::i64(den), _) => ret.extend_from_slice(&[
                 N::r64(Ratio::new(*num as i64, *den as i64))]),
-            (_, _) => unreachable!()
+            _ => unreachable!()
         };
+
+        #[cfg(feature = "num-bigint")]
+        match coeffs {
+            (N::i8(num), N::i8(den), _) => ret.extend_from_slice(&[
+                N::r8(Ratio::new(*num, *den)),
+                N::r16(Ratio::new(*num as i16, *den as i16)),
+                N::r32(Ratio::new(*num as i32, *den as i32)),
+                N::r64(Ratio::new(*num as i64, *den as i64)),
+                N::rbig(Ratio::new((*num).into(), (*den).into()))]),
+            (N::i16(num), N::i16(den), _) => ret.extend_from_slice(&[
+                N::r16(Ratio::new(*num as i16, *den as i16)),
+                N::r32(Ratio::new(*num as i32, *den as i32)),
+                N::r64(Ratio::new(*num as i64, *den as i64)),
+                N::rbig(Ratio::new((*num).into(), (*den).into()))]),
+            (N::i32(num), N::i32(den), _) => ret.extend_from_slice(&[
+                N::r32(Ratio::new(*num as i32, *den as i32)),
+                N::r64(Ratio::new(*num as i64, *den as i64)),
+                N::rbig(Ratio::new((*num).into(), (*den).into()))]),
+            (N::i64(num), N::i64(den), _) => ret.extend_from_slice(&[
+                N::r64(Ratio::new(*num as i64, *den as i64)),
+                N::rbig(Ratio::new((*num).into(), (*den).into()))]),
+            (N::ibig(num), N::ibig(den), _) => ret.extend_from_slice(&[
+                N::rbig(Ratio::new(num.clone(), den.clone()))]),
+            _ => unreachable!()
+        };
+
+        match coeffs.2 {
+            Some(N::f32(v)) => ret.extend_from_slice(&[
+                N::f32(v), N::f64(v as f64)
+            ]),
+            Some(N::f64(v)) => ret.push(N::f64(v as f64)),
+            _ => {}
+        }
         ret
     }
 
     // test comparison and hashing
     for icls in 0..ratio_coeffs.len() {
-        let (inum, iden, ifloat) = &ratio_coeffs[icls];
-        let iequiv = expand_equiv_class_ratio(inum, iden);
+        let iequiv = expand_equiv_class_ratio(&ratio_coeffs[icls]);
 
         // test hashing
         let hashes: Vec<u64> = iequiv.iter().map(hash).collect();
@@ -678,16 +772,126 @@ fn test_rational() {
         }
         
         // test comparison with float
-        if let Some(f) = ifloat {
-            for i in &iequiv {
-                assert_cmp(i, f, Ordering::Equal);
-                assert_eq!(hash(i), hash(f));
-            }
-        }
+        // if let Some(f) = ifloat {
+        //     for i in &iequiv {
+        //         assert_cmp(i, f, Ordering::Equal);
+        //         assert_eq!(hash(i), hash(f), "Hash mismatch between {:?} and {:?}", i, f);
+        //     }
+        // }
 
         for jcls in 0..ratio_coeffs.len() {
-            let (jnum, jden, _) = &ratio_coeffs[jcls];
-            let jequiv = expand_equiv_class_ratio(jnum, jden);
+            let jequiv = expand_equiv_class_ratio(&ratio_coeffs[jcls]);
+
+            let expected = icls.cmp(&jcls);
+            for i in &iequiv {
+                for j in &jequiv {
+                    assert_cmp(i, j, expected);
+                }
+            }
+        }
+    }
+}
+
+#[test]
+#[cfg(feature = "num-complex")]
+fn test_complex_using_primitives() {
+    fn expand_equiv_class_ratio(cls: &[N]) -> Vec<N> {
+        let mut ret = Vec::new();
+    
+        for e in cls {
+            // size extension
+            match e {
+                N::u8(v) => ret.push(N::u8(*v)),
+                N::u16(v) => ret.push(N::u16(*v)),
+                N::u32(v) => ret.push(N::u32(*v)),
+                N::u64(v) => ret.push(N::u64(*v)),
+                N::u128(v) => ret.push(N::u128(*v)),
+                N::i8(v) => ret.push(N::i8(*v)),
+                N::i16(v) => ret.push(N::i16(*v)),
+                N::i32(v) => ret.push(N::i32(*v)),
+                N::i64(v) => ret.push(N::i64(*v)),
+                N::i128(v) => ret.push(N::i128(*v)),
+                N::f64(v) => ret.extend_from_slice(&[N::f64(*v), N::c64(Complex::from(*v))]),
+                N::f32(v) => ret.extend_from_slice(&[N::f32(*v), N::c32(Complex::from(*v)),
+                    N::f64(*v as f64), N::c64(Complex::from(*v as f64))]),
+                _ => {}
+            }
+        }
+        ret
+    }
+
+    let numbers: Vec<_> = NUMBERS.iter().map(|cls| expand_equiv_class_ratio(cls)).collect();
+
+    // comparison between numbers
+    for icls in 0..numbers.len() {
+        for jcls in 0..numbers.len() {
+            let expected = icls.cmp(&jcls);
+            for i in &numbers[icls] {
+                for j in &numbers[jcls] {
+                    assert_cmp(i, j, expected);
+                }
+            }
+        }
+    }
+
+    for &equiv in NUMBERS {
+        let equiv = expand_equiv_class_ratio(equiv);
+        let hashes: Vec<u64> = equiv.iter().map(hash).collect();
+        for i in 1..equiv.len() {
+            assert_eq!(hashes[0], hashes[i], "Hash mismatch between {:?} and {:?}", equiv[0], equiv[i]);
+        }
+    }
+}
+
+#[test]
+#[cfg(feature = "num-complex")]
+fn test_complex() {
+    // additional test cases for complex numbers: (real, image)
+    let ratio_coeffs = [        
+        (N::f32(-1.), N::f32(-1.)),
+        (N::f32(-1.), N::f32(0.)),
+        (N::f32(-1.), N::f32(1.)),
+
+        (N::f32(0.), N::f32(-1.)),
+        (N::f32(0.), N::f32(0.)),
+        (N::f32(0.), N::f32(1.)),
+
+        (N::f32(1.), N::f32(-1.)),
+        (N::f32(1.), N::f32(0.)),
+        (N::f32(1.), N::f32(1.)),
+    ];
+
+    fn expand_equiv_class_ratio(coeffs: &(N, N)) -> Vec<N> {
+        let mut ret = Vec::new();
+        match coeffs {
+            (N::f32(re), N::f32(im)) if im == &0. => ret.extend_from_slice(&[
+                N::c32(Complex::new(*re, *im)),
+                N::c64(Complex::new(*re as f64, *im as f64)),
+                N::f32(*re), N::f64(*re as f64)]),
+            (N::f32(re), N::f32(im)) => ret.extend_from_slice(&[
+                N::c32(Complex::new(*re, *im)),
+                N::c64(Complex::new(*re as f64, *im as f64))]),
+            (N::f64(re), N::f64(im)) if im == &0. => ret.extend_from_slice(&[
+                N::c64(Complex::new(*re, *im)), N::f64(*re as f64)]),
+            (N::f64(re), N::f64(im)) => ret.push(
+                N::c64(Complex::new(*re, *im))),
+            (_, _) => unreachable!()
+        };
+        ret
+    }
+
+    // test comparison and hashing
+    for icls in 0..ratio_coeffs.len() {
+        let iequiv = expand_equiv_class_ratio(&ratio_coeffs[icls]);
+
+        // test hashing
+        let hashes: Vec<u64> = iequiv.iter().map(hash).collect();
+        for i in 1..iequiv.len() {
+            assert_eq!(hashes[0], hashes[i], "Hash mismatch between {:?} and {:?}", iequiv[0], iequiv[i]);
+        }
+
+        for jcls in 0..ratio_coeffs.len() {
+            let jequiv = expand_equiv_class_ratio(&ratio_coeffs[jcls]);
 
             let expected = icls.cmp(&jcls);
             for i in &iequiv {
